import SkribblClient from "../logic/SkribblClient.js";
import LatexJS from "../util/Latex.js";
import Util, {CustomElement} from "../util/Util.js";
import SkribblCanvas from "./SkribblCanvas.js";

/**
 * Custom element `<skribbl-canvas-container>` that manages the canvas and its overlays.
 */
export default class SkribblCanvasContainer extends CustomElement {
	/**
	 * @param {SkribblClient} client 
	 */
	constructor(client){
		super();
		this.attachShadow({mode:"open"});
		this.shadowRoot.innerHTML = `
			<style>
				:host {
					display: block;
					position: relative;
				}
				svg {
					display: block;
					width: 100%;
				}
				#container {
					position: absolute;
					left: 0;
					top: 0;
					width: 100%;
					height: 100%;
					overflow: hidden;
					background: #ffffff;
					border-radius: 5px;
				}
				#controls {
					position: absolute;
					top: 100%;
					margin-top: 10px;
				}
				#controls.hidden {
					display: none;
				}
				.button {
					margin: auto;
					text-align: center;
					
					display:inline-block;
					border:none;
					height:40px;
					width:40px;
					
					/*
					background: url('/res/selectArrows.png');
					background-size: 54px;
					background-repeat: no-repeat;*/

					cursor: pointer;
				}
				.colorButton{
					height: 20px;
					width: 20px;	
					}
				.widthButton{
					height:40px;
					width: 40px;
				}
				.drawButton{
					height: 40px;
					width: 100px;
				}

			</style>
			<!-- invisible svg to maintain an aspect ratio of 960:720, aka 4:3 -->
			<svg viewBox="0 0 960 720"></svg>
			<div id="container">
				<slot name="overlay"></slot>
			</div>
			<div id="controls">
				<!--Color settings & stuff goes here.-->
				<div id = "darkred" class = "button colorButton" style="background: DarkRed"> </div>
				<div id = "red" class = "button colorButton" style="background: Firebrick"> </div>
				<div id = "orange" class = "button colorButton" style="background: orange"> </div>
				<div id = "yellow" class = "button colorButton" style="background: yellow"> </div>
				<div id = "lightgreen" class = "button colorButton" style="background: YellowGreen"> </div>
				<div id = "green" class = "button colorButton" style="background: ForestGreen"> </div>
				<div id = "lightblue" class = "button colorButton" style="background: SkyBlue"> </div>
				<div id = "blue" class = "button colorButton" style="background: SlateBlue"> </div>
				<div id = "purple" class = "button colorButton" style="background: BlueViolet"> </div>
				<div id = "pink" class = "button colorButton" style="background: pink"> </div>
				<div id = "lightbrown" class = "button colorButton" style="background: SandyBrown"> </div>
				<div id = "darkbrown" class = "button colorButton" style="background: SaddleBrown"> </div>
				<div id = "lightgrey" class = "button colorButton" style="background: LightGrey"> </div>
				<div id = "darkgrey" class = "button colorButton" style="background: DarkGrey"> </div>
				<div id = "black" class = "button colorButton" style="background: black"> </div>
				<div id = "white" class = "button colorButton" style="background: white"> </div>

				
				<div id = "widthS" class = "button widthButton" style="background: white"> dünn </div>
				<div id = "widthM" class = "button widthButton" style="background: white"> mittel </div>
				<div id = "widthL" class = "button widthButton" style="background: white">  dick </div>
				<div id = "widthXL" class = "button widthButton" style="background: white"> sehr dick  </div>
				 
				<div id = "pen" class = "button drawButton" style="background:white"> malen  </div>
				<div id = "erasor" class = "button drawButton" style="background: white"> radieren </div>
				<div id = "changeBackgroundColor" class = "button drawButton" style="background: white"> Hintergrund färben </div>
				
			</div>
		`;


		this._canvas = new SkribblCanvas(client);
		this._container = this.shadowRoot.getElementById("container");
		this._container.insertBefore(this._canvas,this._container.firstChild);
		this._controls = this.shadowRoot.getElementById("controls");
		client.state.onChange(state=>{
			/** whether the player should currently be allowed to draw on the canvas */
			let allowToDraw = state.hasGameStarted&&state.drawingPlayer==state.playerIndex&&typeof state.word=="string";
			this._canvas.drawable = allowToDraw;
			this._controls.classList.toggle("hidden",!allowToDraw);
		},{onlyWhen:this.connected});
		// TODO implement color settings and set the correspondig properties of `this._canvas` whenever they change here. 

		//Logik fuer die Buttons zur Farbauswahl
		this._darkredButton = this.shadowRoot.querySelector("#darkred");
		this._darkredButton.addEventListener("click",e=>{
			this._canvas.color = "DarkRed";		
		});
		this._redButton = this.shadowRoot.querySelector("#red");
		this._redButton.addEventListener("click",e=>{
			this._canvas.color = "Firebrick";	
		});
		this._orangeButton = this.shadowRoot.querySelector("#orange");
		this._orangeButton.addEventListener("click",e=>{
			this._canvas.color = "orange";	
		});
		this._yellowButton = this.shadowRoot.querySelector("#yellow");
		this._yellowButton.addEventListener("click",e=>{
			this._canvas.color = "yellow";		
		});
		this._lightgreenButton = this.shadowRoot.querySelector("#lightgreen");
		this._lightgreenButton.addEventListener("click",e=>{
			this._canvas.color = "YellowGreen";
		});
		this._greenButton = this.shadowRoot.querySelector("#green");
		this._greenButton.addEventListener("click",e=>{
			this._canvas.color = "ForestGreen";			
		});
		this._lightblueButton = this.shadowRoot.querySelector("#lightblue");
		this._lightblueButton.addEventListener("click",e=>{
			this._canvas.color = "SkyBlue";			
		});
		this._blueButton = this.shadowRoot.querySelector("#blue");
		this._blueButton.addEventListener("click",e=>{
			this._canvas.color = "SlateBlue";
		});
		this._pinkButton = this.shadowRoot.querySelector("#pink");
		this._pinkButton.addEventListener("click",e=>{
			this._canvas.color = "pink";	
		});
		this._purpleButton = this.shadowRoot.querySelector("#purple");
		this._purpleButton.addEventListener("click",e=>{
			this._canvas.color = "BlueViolet";		
		});
		this._pinkButton = this.shadowRoot.querySelector("#pink");
		this._pinkButton.addEventListener("click",e=>{
			this._canvas.color = "pink";	
		});
		this._lightbrownButton = this.shadowRoot.querySelector("#lightbrown");
		this._lightbrownButton.addEventListener("click",e=>{
			this._canvas.color = "SandyBrown";		
		});
		this._darkbrownButton = this.shadowRoot.querySelector("#darkbrown");
		this._darkbrownButton.addEventListener("click",e=>{
			this._canvas.color = "SaddleBrown";		
		});
		this._lightgreyButton = this.shadowRoot.querySelector("#lightgrey");
		this._lightgreyButton.addEventListener("click",e=>{
			this._canvas.color = "LightGrey";		
		});
		this._darkgreyButton = this.shadowRoot.querySelector("#darkgrey");
		this._darkgreyButton.addEventListener("click",e=>{
			this._canvas.color = "DarkGrey";	
		});
		this._blackButton = this.shadowRoot.querySelector("#black");
		this._blackButton.addEventListener("click",e=>{
			this._canvas.color = "black";	
		});
		this._whiteButton = this.shadowRoot.querySelector("#white");
		this._whiteButton.addEventListener("click",e=>{
			this._canvas.color = "white";	
		});

		//Logik fuer Buttons zur Auswahl der Stiftdicke
		this._widthSDrawButton = this.shadowRoot.querySelector("#widthS");
		this._widthSDrawButton.addEventListener("click",e=>{
			this._canvas.radius = 10;		
		});
		this._widthMDrawButton = this.shadowRoot.querySelector("#widthM");
		this._widthMDrawButton.addEventListener("click",e=>{
			this._canvas.radius = 20;		
		});
		this._widthLDrawButton = this.shadowRoot.querySelector("#widthL");
		this._widthMDrawButton.addEventListener("click",e=>{
			this._canvas.radius = 30;		
		});
		this._widthXLDrawButton = this.shadowRoot.querySelector("#widthXL");
		this._widthXLDrawButton.addEventListener("click",e=>{
			this._canvas.radius = 50;		
		});

		//Logik fuer Buttons (Radieren und Malen)
		this._penButton = this.shadowRoot.querySelector("#pen");
		this._penButton.addEventListener("click",e=>{
			this._canvas.penActive = true;		
		});
		this._erasorButton = this.shadowRoot.querySelector("#erasor");
		this._erasorButton.addEventListener("click",e=>{
			this._canvas.erasorActive = true;		
		});

	}

	set overlay(overlay){
		if (this._overlay){
			this._overlay.remove();
		}
		this._overlay = overlay;
		if (overlay){
			overlay.slot = "overlay";
			this.appendChild(overlay);
		}
	}

	/**
	 * @type {SkribblCanvasOverlay}
	 */
	get overlay(){
		return this._overlay;
	}
}
/**
 * Custom element `<skribbl-canvas-overlay>` for overlays on the `<skribbl-canvas>`-element.
 */
export class SkribblCanvasOverlay extends HTMLElement {
	/**
	 * Constructs a new SkribblCanvasOverlay and appends the given nodes or strings to it.
	 * @param {(string|Node)[]|import("../logic/SkribblClient.js").WordReveal} data the nodes or data to display
	 * @param {SkribblClient} client
	 */
	constructor(data=[],client=null){
		super();
		this.attachShadow({mode:"open"});
		this.shadowRoot.innerHTML = `
			<style>
				:host {
					display: block;
					position: absolute;
					top: 0;
					right: 0;
					bottom: 0;
					left: 0;
					background: #000000bf;
					color: #ffffff;
				}
				#outer {
					position: absolute;
					left: 50%;
					top: 50%;
					width: 100%;
					height: 100%;
				}
				#contents {
					position: absolute;
					max-width: 80%;
					transform: translate(-50%,-50%);
					text-align: center;
					font-size: 1.2em;
				}
				::slotted(button) {
					padding: 3px 5px;
					margin: 3px;
					border: none;
					border-radius: 3px;
					background: #dfdfdf;
					color: #000000;
					font-size: 1em;
				}
				::slotted(button:hover) {
					cursor: pointer;
				}
				table {
					display: inline-block;
					font-size: 1.1em;
				}
				td {
					padding: 1px 8px;
				}
				td:first-child {
					text-align: left;
				}
				td:last-child {
					text-align: right;
				}
				h2+latex-js.description {
					display: block;
					margin-top: -1em;
					margin-bottom: 1em;
				}
			</style>
			<div id="outer">
				<div id="contents">
					<slot></slot>
				</div>
			</div>
		`;
		this._contents = this.shadowRoot.querySelector("#contents");
		if (Array.isArray(data)){
			this.append(...data);
		}else{
			let points = data.points.map((points,index)=>({name:client.players.value[index].name,points}));
			points.sort((a,b)=>b.points-a.points);
			this._contents.innerHTML = `
				<h2>The word was: ${Util.htmlEscape(data.word)}</h2>
				<table>
					${points.map(({name,points})=>`
						<tr>
							<td>${Util.htmlEscape(name)}:</td>
							<td style="color:${points>0?"#00bf00":"#bf0000"}">+${Util.htmlEscape(points.toString())}</td>
						</tr>
					`).join("")}
				</table>
			`;
			if (data.description){
				let latex = new LatexJS(data.description,{macros:data.macros});
				latex.className = "description";
				this._contents.insertBefore(latex,this._contents.children[1]);
			}
		}
	}
}
customElements.define("skribbl-canvas-container",SkribblCanvasContainer);
customElements.define("skribbl-canvas-overlay",SkribblCanvasOverlay);